package com.hive.communication.netty.server;

import com.hive.communication.netty.server.decoder.DelimiterBasedFrameDecoder;
import com.hive.communication.netty.server.handler.EchoServerHandler;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.bytes.ByteArrayEncoder;
import io.netty.handler.codec.string.StringEncoder;

import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.Map;

public class EchoServer {

    private final  int port;

    public EchoServer(int port) {
        this.port = port;
    }

    private static final Map<Integer, Channel> portToChannelMap = new HashMap<>();
    private static final Map<Integer, ServerBootstrap> portToBootstrapMap = new HashMap<>();
    private static final Map<Integer, EventLoopGroup> portToEventLoopGroupMap = new HashMap<>();

    public void start() {
                EventLoopGroup bossGroup = new NioEventLoopGroup();
                EventLoopGroup group = new NioEventLoopGroup();
                try {
                    ServerBootstrap sb = new ServerBootstrap();
                    sb.option(ChannelOption.SO_BACKLOG, 1024);
                    sb.group(group, bossGroup) // 绑定线程池
                            .channel(NioServerSocketChannel.class) // 指定使用的channel
                            .localAddress(port)// 绑定监听端口
                            //保持连接数
                            .option(ChannelOption.SO_BACKLOG, 128)
                            //有数据立即发送
                            .option(ChannelOption.TCP_NODELAY, true)
                            //保持连接
                            .childOption(ChannelOption.SO_KEEPALIVE, true)
                            //处理新连接
                            .childHandler(new ChannelInitializer<SocketChannel>() { // 绑定客户端连接时候触发操作

                                @Override
                                protected void initChannel(SocketChannel ch) throws Exception {
                                    System.out.println("报告");
                                    System.out.println("信息：有一客户端链接到本服务端");
                                    System.out.println("IP:" + ch.localAddress().getHostName());
                                    System.out.println("Port:" + ch.localAddress().getPort());
                                    System.out.println("报告完毕");

                                    ch.pipeline().addLast(new StringEncoder(Charset.forName("UTF-8")));
                                    String startDelimiterStr = "99";
                                    String endDelimiterStr = "0A";
                                    ByteBuf startDelimiter = Unpooled.copiedBuffer(startDelimiterStr, Charset.forName("UTF-8"));
                                    ByteBuf endDelimiter = Unpooled.copiedBuffer(endDelimiterStr, Charset.forName("UTF-8"));
//                                    ch.pipeline().addLast(new DelimiterBasedFrameDecoder(startDelimiter, endDelimiter));
                                    ch.pipeline().addLast(new EchoServerHandler());
                                    ch.pipeline().addLast(new ByteArrayEncoder());

                                }
                            });
                    ChannelFuture cf = sb.bind().sync(); // 服务器异步创建绑定
                    System.out.println(EchoServer.class + " 启动正在监听： " + cf.channel().localAddress());
                    portToChannelMap.put(port, cf.channel());
                    portToBootstrapMap.put(port, sb);
                    portToEventLoopGroupMap.put(port, group);
                    cf.channel().closeFuture().sync(); // 关闭服务器通道
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    //释放线程池资源
                    group.shutdownGracefully();
                    bossGroup.shutdownGracefully();
                }
    }

    public void stopServer(int port) {
        Channel channel = portToChannelMap.get(port);
        if (channel != null && channel.isOpen()) {
            ChannelFuture future = channel.close();
            future.addListener(f -> {
                if (f.isSuccess()) {
                    portToChannelMap.remove(port);
                    // 确保在Channel关闭后处理ServerBootstrap和EventLoopGroup
                    handleServerBootstrapAndEventLoopGroup(port);
                } else {
                    // 处理关闭Channel失败的情况
                }
            });
        } else {
            // 处理Channel不存在或已关闭的情况
            handleServerBootstrapAndEventLoopGroup(port);
        }
    }

    private void handleServerBootstrapAndEventLoopGroup(int port) {
        ServerBootstrap bootstrap = portToBootstrapMap.get(port);
        if (bootstrap != null) {
            EventLoopGroup bossGroup = portToEventLoopGroupMap.get(port);
            EventLoopGroup workerGroup = bootstrap.childGroup();
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
            portToBootstrapMap.remove(port);
            portToEventLoopGroupMap.remove(port);
        } else {
            System.out.println("找不到端口" + port);
        }
    }


}