package com.hive.communication.netty.server.dataprocessing;

import cn.hutool.core.util.HexUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.hive.bee.entity.BeeHive;
import com.hive.communication.util.HexConversion;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

import static com.hive.communication.netty.server.handler.EchoServerHandler.sendActive;
import static com.hive.util.DateConvetUtil.isDifferenceMoreThan10Minutes;

/**
 * 数据处理
 *
 * @Author: hive - LGH
 * @Date: 2024/10/11 14:46
 * @Version: V1.0
 */
@Slf4j
@Component
public class HandlerDateProcessing {


    /**
     * 处理响应数据
     */
    private final AtomicBoolean isScheduled = new AtomicBoolean(false);

    // private boolean isScheduled = false;
    public void handleResponse(ChannelHandlerContext ctx, StringBuffer sb) throws Exception {
        boolean flag = false;
        log.info("收到蜂箱数据" + sb);
        BeeHive beeHive = dataAnalysis(sb.toString());
        if (ObjectUtils.isNotNull(beeHive)) {
            BeeHiveAdd beeHiveAdd = SpringUtil.getBean(BeeHiveAdd.class);
            beeHiveAdd.addAndEdit(beeHive);
            flag = true;
        }
        // 处理响应逻辑
        // 读取数据并获取设备编号
        // String deviceId = deviceChannels.get(ctx.channel().id());
        // String deviceId = ctx.channel().attr(DEVICE_ID).get();
        //数据长度不够则解析异常，需要设备再次上报
     /*   if (!flag) {
            //解析 数据不符合要求，需要设备再次上报
        } else {

            //变更注册回复协议
        }*/
        // 创建一个CompletableFuture，将在2s后完成
        if (flag) {
            CompletableFuture<Void> future = new CompletableFuture<>();
            //isScheduled.compareAndSet 确保schedule()方法只被调用一次
            if (isScheduled.compareAndSet(false, true)) {
                ctx.channel().eventLoop().schedule(() -> {
                    try {
                        // 正常完成Future
                        future.complete(null);
                    } catch (Exception e) {
                        // 异常完成Future
                        future.completeExceptionally(e);
                    } finally {
                        isScheduled.set(false); // 无论如何都将标志重置为false
                    }
                    // }, 4, TimeUnit.HOURS);
                }, 2, TimeUnit.SECONDS);  // 将延迟时间设置为2s
                //  isScheduled = true; // 设置标志变量为true，表示延迟逻辑已被调度
                future.thenAccept(v -> sendActive(ctx, true, beeHive, true));
                //ctx.writeAndFlush(messageBuffer)

            } else {
                //ctx.writeAndFlush();
            }
        }
    }

    public BeeHive dataAnalysis(String data) {
        BeeHiveAdd beeHiveAdd = SpringUtil.getBean(BeeHiveAdd.class);
        BeeHive beeHive = new BeeHive();
        if (data.length() != 61) {
            return beeHive;
        }
        for (int i = 0; i < 7; i++) {
            Integer startIndex = 0;
            Integer endIndex = 0;
            switch (i) {
                case 0:
                    //蜂箱id
                    startIndex += 16;
                    endIndex += 32;
                    String subData = getSubData(data, startIndex, endIndex);
                    beeHive = beeHiveAdd.getOneBeeHive(subData);
                    if (ObjectUtils.isNull(beeHive)) {
                        return null;
                    }
                    beeHive.setDeviceId(subData);
                    break;
                case 1:
                    //时间戳
                    startIndex += 6;
                    endIndex += 14;
                    long date = HexConversion.hexToBigInt(getSubData(data, startIndex, endIndex));
                    //时间戳大于10分钟则为有效数据
//                    if (!isDifferenceMoreThan10Minutes(Instant.now().getEpochSecond(), date)) {
//                        return null;
//                    }
                    beeHive.setDateUploadTime(LocalDateTime.ofInstant(Instant.ofEpochSecond(date), ZoneId.systemDefault()));
                    break;
                case 2:
                    //当前温度值2
                    startIndex += 36;
                    endIndex += 40;
                    int i2 = HexUtil.hexToInt(getSubData(data, startIndex, endIndex));
                    beeHive.setCurrentTemperature(String.valueOf(new BigDecimal(i2).multiply(new BigDecimal("0.1"))));
                    //startIndex=4 ,endIndex=10
                    break;
                case 3:
                    //当前湿度值2
                    startIndex += 40;
                    endIndex += 44;
                    beeHive.setCurrentHumidity(String.valueOf(HexConversion.hexadecimal16Conversion(getSubData(data, startIndex, endIndex))));
                    //startIndex=4 ,endIndex=10
                    break;
                case 4:
                    //蜂箱重量4
                    startIndex += 44;
                    endIndex += 52;
                    int i1 = HexUtil.hexToInt(getSubData(data, startIndex, endIndex));
                    beeHive.setBeeHiveWeight(String.valueOf(BigDecimal.valueOf(i1).multiply(BigDecimal.valueOf(0.001))));
                    //startIndex=4 ,endIndex=10
                    break;
                case 5:
                    //蜂进出数量4
                    startIndex += 52;
                    endIndex += 60;
                    beeHive.setCurrentNumber(String.valueOf(HexUtil.hexToInt(getSubData(data, startIndex, endIndex))));
                    //startIndex=4 ,endIndex=10
                    break;
                case 6:
                    //声音异常0表示无异常FF表示异常1
                    startIndex += 60;
                    endIndex += 62;
                    beeHive.setCurrentNoise(String.valueOf(getSubData(data, startIndex, endIndex)));
                    break;
            }
        }
        return beeHive;
    }

    public String getSubData(String data, Integer startIndex, Integer endIndex) {
        return data.substring(startIndex, endIndex);
    }

}
