package com.hive.communication.netty.server.decoder;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

import java.util.List;

/**
 * @Author: HIVE - LGH
 * @Date: 2024/10/11 13:27
 * @Version: V1.0
 */
public class CustomDecoder  extends ByteToMessageDecoder {
    private static final byte HEARTBEAT_FLAG = (byte) 0xfe; // 心跳消息的标识
    private int length1;
    private int length2;
    private byte specialFlag;

    public CustomDecoder(int length1, int length2, byte specialFlag) {
        this.length1 = length1;
        this.length2 = length2;
        this.specialFlag = specialFlag;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        // 检查是否为心跳消息
        // 如果是心跳消息，则将其添加到输出列表中
        out.add(in.readBytes(in.readableBytes()));

        if (in.getByte(in.readerIndex()) == HEARTBEAT_FLAG) {
            // 如果是心跳消息，则将其添加到输出列表中
            out.add(in.readBytes(in.readableBytes()));
        }else {
            // 检查特殊标识
            if (in.getByte(in.readerIndex()) == specialFlag) {
                // 解析长度为length1的消息(水文协议)
                if (in.readableBytes() >= length1) {
                    out.add(in.readBytes(length1));
                }
            } else {
                // 解析长度为length2的消息(modbus协议)
                if (in.readableBytes() >= length2) {
                    out.add(in.readBytes(length2));
                }/*else{
                    // 不满足长度，释放已读取的ByteBuf
                    ReferenceCountUtil.release(in);
                }*/
            }
        }
    }
  /*  private int length; // 消息长度

    public CustomDecoder(int length) {
        this.length = length;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
        if (in.readableBytes() >= length) {
            out.add(in.readBytes(length)); // 将指定长度的数据添加到输出列表中
        }


    }*/


}
