package com.hive.communication.netty.server.decoder;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;

import java.util.List;

/**
 * @Author: HIVE - LGH
 * @Date: 2024/10/11 15:42
 * @Version: V1.0
 */
public class DelimiterBasedFrameDecoder extends ByteToMessageDecoder {

    private final ByteBuf startDelimiter; // 开始标识字符串
    private final ByteBuf endDelimiter;   // 结束标识字符串

    public DelimiterBasedFrameDecoder(ByteBuf startDelimiter, ByteBuf endDelimiter) {
        this.startDelimiter = startDelimiter;
        this.endDelimiter = endDelimiter;
    }

    @Override
    protected void decode(ChannelHandlerContext ctx, ByteBuf in, List<Object> out) throws Exception {
     /*   // 将ByteBuf转换为Java字符串以便搜索标识符
        String data = in.toString(StandardCharsets.US_ASCII);

        int startIndex = data.indexOf(startDelimiter);
        int endIndex = data.indexOf(endDelimiter);

        if (startIndex != -1 && endIndex != -1 && startIndex < endIndex) {
            String frame = data.substring(startIndex + startDelimiter.length(), endIndex);
            out.add(frame); // 添加提取的数据帧到输出列表中
        }*/

        // 将ByteBuf转换为byte数组
        byte[] startDelim = new byte[startDelimiter.readableBytes()];
        startDelimiter.getBytes(startDelimiter.readerIndex(), startDelim);

        byte[] endDelim = new byte[endDelimiter.readableBytes()];
        endDelimiter.getBytes(endDelimiter.readerIndex(), endDelim);

        int startDelimIndex = ByteBufUtil.indexOf(startDelimiter, in);
        int endDelimIndex = ByteBufUtil.indexOf(endDelimiter, in);

        // 确保找到了开始和结束分隔符
        if (startDelimIndex >= 0 && endDelimIndex > startDelimIndex) {
            // 提取数据帧
            ByteBuf frame = in.slice(startDelimIndex + startDelimiter.readableBytes(), endDelimIndex - (startDelimIndex + startDelimiter.readableBytes()));
            // 将String转换为ByteBuf，使用UTF-8编码
            out.add(frame.retain());
            // 更新readerIndex到结束分隔符之后
            in.readerIndex(endDelimIndex + endDelimiter.readableBytes());
        }
    }


}
