package com.hive.communication.netty.server.handler;


import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.hive.bee.entity.BeeHive;
import com.hive.communication.netty.server.dataprocessing.HandlerDateProcessing;
import com.hive.communication.util.HexConversion;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.*;
import io.netty.handler.timeout.ReadTimeoutException;
import io.netty.util.AttributeKey;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import java.io.UnsupportedEncodingException;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.*;

import static com.hive.util.DateConvetUtil.getCurrentSeconds;

@Slf4j
public class EchoServerHandler extends ChannelInboundHandlerAdapter {


    private static final String HEARTBEAT_REQUEST = "FE";
    //private final int reconnectDelay; // 重连延迟时间，单位为秒
    private boolean isHeartbeatReceived = false;
    // 创建一个固定大小的线程池用于消息处理
    //  private final ExecutorService executorService = Executors.newFixedThreadPool(10);
    private static final ScheduledExecutorService scheduler = Executors.newScheduledThreadPool(10);
    // 使用AttributeKey定义设备编号属性的键
    private static final AttributeKey<String> DEVICE_ID = AttributeKey.valueOf("deviceId");
    private static final ConcurrentHashMap<String, ChannelPromise> pendingRequests = new ConcurrentHashMap<>();

    /*****************************************  发送消息  *****************************************/
    /**
     * channelAction
     * channel 通道 action 活跃的
     * 当客户端主动链接服务端的链接后，这个通道就是活跃的了。也就是客户端与服务端建立了通信通道并且可以传输数据
     */

    public static void  sendActive(ChannelHandlerContext ctx, boolean flag, BeeHive beeHive, boolean notice) {
        if (flag) {
                            try {
                                // String paddedString = String.format("%4s", code).replace(' ', '0');
                                String codes = "注册完回复";
                                //获取十分钟后的时间秒数，需要转换为16进制
                                getCurrentSeconds();
                                log.info("当前时间: {}, 消息: {}", LocalDateTime.now(), codes);
                                // 先将十六进制字符串转换为字节数组
                                byte[] messageBytes = HexConversion.hexStringToByteArray(codes);
                                // 将字节数组封装到ByteBuf中
                                ByteBuf messageBuffer = Unpooled.wrappedBuffer(messageBytes);
                                // 通过ChannelHandlerContext发送ByteBuf
                                // 关联设备编号与Channel
                                //   sendRequest(ctx.channel(),messageBuffer,devicePointsSensor.getId());
                                retrySend(ctx, messageBuffer)
                                        .whenComplete((result,error) -> {
                                            log.info("发送状态: {},当前时间: {}, 消息: {}，名称：{}，蜂箱id：{}", result,LocalDateTime.now(), codes,beeHive.getBeeHiveName(),beeHive.getId());
                                       /*      if (result) {
                                                // 发送成功
                                            } else {
                                                // 发送失败
                                            }*/
                                        });
                            } catch (Exception e) {
                                // 处理异常
                            }
        }
    }

    /**
     * 功能：调用retrySendAsync并返回结果
     */
    public static CompletableFuture<Boolean> retrySend(ChannelHandlerContext ctx, ByteBuf messageBuffer) {
        int maxRetries = 3; // 最大重试次数
        CompletableFuture<Boolean> future = new CompletableFuture<>();
        try {
            retrySendAsync(ctx, messageBuffer, maxRetries, future);
        } catch (Exception e) {
            //确保异常程序正常下行
            e.printStackTrace();
            //释放资源
            if (messageBuffer.refCnt() > 0) {
                messageBuffer.release();
            }
            future.complete(false);
        }
        return future;
    }
    /**
     * 功能：发送消息并监听成功与失败进行重试
     */
    private static void retrySendAsync(ChannelHandlerContext ctx, ByteBuf messageBuffer, int remainingRetries, CompletableFuture<Boolean> future) {
        messageBuffer.retain(); // 增加引用计数
        ChannelFuture sendFuture = ctx.writeAndFlush(messageBuffer);
        sendFuture.addListener((ChannelFutureListener) sendFutureListener -> {
                if (sendFutureListener.isSuccess()) {
                    messageBuffer.release(); // 释放在发送前增加的引用计数
                    future.complete(true); // 发送成功，将结果设置为 true
                } else {
                    if (remainingRetries > 0) {
                        ctx.executor().schedule(() -> {
                        // 发送失败，但还有剩余重试次数
                        if (messageBuffer.isReadable()) {
                            // 如果messageBuffer仍然可读，重新使用它进行重试
                            retrySendAsync(ctx,messageBuffer, remainingRetries - 1, future);

                        } else {
                            // messageBuffer不可读，需要重新创建或复制一个新的messageBuffer
                            ByteBuf newMessageBuffer = messageBuffer.retainedDuplicate();
                            retrySendAsync(ctx, newMessageBuffer, remainingRetries - 1, future);
                            }
                        }, 2, TimeUnit.SECONDS);
                    } else {
                        messageBuffer.release(); // 释放在发送前增加的引用计数
                        future.complete(false); // 达到最大重试次数，将结果设置为 false

                    }
                }
        });
    }
    /**
     * 功能：或缺设备编号等存入pendingRequests等读取是进行匹配
     */
    public static ChannelFuture sendRequest(Channel channel, ByteBuf message, String messageId) {
        ChannelPromise promise = channel.newPromise();
        pendingRequests.put(messageId, promise);
        // 将messageId附加到消息中，并发送消息
        // 这里用逗号分隔messageId和消息体
        ByteBuf msgToSend = message.copy().writeBytes((messageId + ",").getBytes(CharsetUtil.UTF_8));
        channel.writeAndFlush(msgToSend).addListener((ChannelFutureListener) future -> {
            if (!future.isSuccess()) {
                promise.setFailure(future.cause());
                pendingRequests.remove(messageId);
            }
        });

        return promise;
    }
    /**
     * 功能：连接成功调用
     */
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        //ChannelManager.addChannel(ctx.channel(),(Integer) localAddress(ctx).get("port"));
        //NettyConfig.group.add(ctx.channel());
    }

    /**
     * channelInactive
     * channel 通道 Inactive 不活跃
     * 当客户端主动断开服务端的链接后，这个通道就是不活跃的。也就是说客户端与服务端的关闭了通信通道并且不可以传输数据
     */
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        System.out.println(ctx.channel().localAddress().toString() + " 通道不活跃！");
        // 关闭流  只将当前的Channel从ChannelGroup中移除，不会立即关闭Channel
        // NettyConfig.group.remove(ctx.channel());
        //关闭当前的Channel并释放相关资源
        ctx.close();
    }

    /**
     * @param buf
     * @return
     * @author Taowd
     * TODO  此处用来处理收到的数据中含有中文的时  出现乱码的问题
     */
    private String getMessage(ByteBuf buf) {
        byte[] con = new byte[buf.readableBytes()];
        buf.readBytes(con);
        try {
            return new String(con, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
            return null;
        }
    }
     /*****************************************处理收到的消息  *****************************************/

    /**
     * 功能：读取发送过来的信息
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        if (ObjectUtils.isNotNull(msg)) ;
        // StringBuffer sb = null;
        ByteBuf buf = (ByteBuf) msg;
        byte[] req = new byte[buf.readableBytes()];
        buf.readBytes(req);
        // String body = new String(req, "UTF-8");
        StringBuffer sb = new StringBuffer(req.length * 2);
        for (int i = 0; i < req.length; i++) {
            sb.append(Character.forDigit((req[i] & 240) >> 4, 16));
            sb.append(Character.forDigit(req[i] & 15, 16));
        }
        // System.out.println("接收到的数据sb" + sb.toString());
        // 检查消息是否是心跳请求
        if (sb.toString().equalsIgnoreCase(HEARTBEAT_REQUEST)) {
            handleHeartbeat(ctx,msg);
        } else {
            // 如果不是心跳请求，调用ChannelInboundHandler的下一个处理器
            ctx.fireChannelRead(msg);
            if (ObjectUtils.isNotNull(sb) && sb.length() > 4) {
                new HandlerDateProcessing().handleResponse(ctx, sb);
            }
        }
        //  System.out.println("客户端收到服务器数据111 服务端ip与端口:" + localAddress(ctx));
        //  System.out.println("客户端收到服务器数据111 客户端ip与端口:" + remoteAddress(ctx));
    }

    private void handleHeartbeat(ChannelHandlerContext ctx, Object msg) {
        isHeartbeatReceived = true;
         ctx.fireChannelRead(msg);
        //ctx.fireChannelReadComplete();
        // 如果需要，发送心跳响应
        //  ctx.writeAndFlush("FE");
    }

    /**
     * 功能：读取完毕客户端发送过来的数据之后的操作
     */
    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        // 第一种方法：写一个空的buf，并刷新写出区域。完成后关闭sock channel连接。
        ctx.writeAndFlush(Unpooled.EMPTY_BUFFER);
        //ctx.flush();
        // 调用父处理器的channelReadComplete方法，确保事件能传递到下一个处理器
        super.channelReadComplete(ctx);
        // 在任何情况下都传递到下一个处理器
        //ctx.fireChannelReadComplete();
        // ctx.flush();
        // 第二种方法：在client端关闭channel连接，这样的话，会触发两次channelReadComplete方法。
        // ctx.flush().close().sync(); // 第三种：改成这种写法也可以，但是这中写法，没有第一种方法的好。
    }

    /**
     * 功能：服务端发生异常的操作
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
     /*   cause.printStackTrace();
        ctx.close();*/
        if (cause instanceof ReadTimeoutException) {
            // 处理超时情况，例如关闭连接或者记录日志等
            ctx.close();
        } else {
            // 其他异常情况的处理
            cause.printStackTrace();
            ctx.close();
        }
    }

    /**
     * 功能：获取客户端端口与ip
     */
    public static Map remoteAddress(ChannelHandlerContext ctx) {
        SocketAddress clientAddress = ctx.channel().remoteAddress();
// 这里可以进行类型转换，因为通常情况下远程地址是 InetSocketAddress 类型
        Map map = new HashMap();
        if (clientAddress instanceof InetSocketAddress) {
            InetSocketAddress inetSocketAddress = (InetSocketAddress) clientAddress;
            String ipAddress = inetSocketAddress.getAddress().getHostAddress();
            int port = inetSocketAddress.getPort();
            map.put("ipAddress", ipAddress);
            map.put("port", port);
            return map;
        }
        return null;
    }

    /**
     * 功能：获取服务端端口与ip
     */
    public static Map localAddress(ChannelHandlerContext ctx) {
        SocketAddress serverAddress = ctx.channel().localAddress();
        Map map = new HashMap();
        if (serverAddress instanceof InetSocketAddress) {
            InetSocketAddress inetSocketAddress = (InetSocketAddress) serverAddress;
            String ipAddress = inetSocketAddress.getAddress().getHostAddress();
            int port = inetSocketAddress.getPort();
            map.put("ipAddress", ipAddress);
            map.put("port", port);
            return map;
        }
        return null;
    }


}
