package com.hive.communication.netty.server.heartbeat;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

/**
 * @Author: HIVE - LGH
 * @Date: 2024/10/11 15:24
 * @Version: V1.0
 */
public class GatewayHeartbeatHandler  extends IdleStateHandler {

    // 用于记录网关的在线状态
    private boolean isOnline = false;
    // 用于记录上一次设置为在线状态的时间戳
    private long lastOnlineTimestamp = System.currentTimeMillis();

    private static final long ONLINE_STATUS_INTERVAL = TimeUnit.HOURS.toMillis(1); // 1小时

    public GatewayHeartbeatHandler(int readerIdleTimeSeconds) {
        super(readerIdleTimeSeconds, 0, 0, TimeUnit.SECONDS);
    }

    @Override
    protected void channelIdle(ChannelHandlerContext ctx, IdleStateEvent evt) throws Exception {
        if (evt.state() == IdleState.READER_IDLE) {
            // 如果心跳超时，设置为离线，并关闭连接
            setOffline(ctx);
        }
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        if (isHeartbeatMessage(msg)) {
            // 接收到心跳
          //  long currentTime = System.currentTimeMillis();
            // || (currentTime - lastOnlineTimestamp > ONLINE_STATUS_INTERVAL)
            if (!isOnline) {
                // 如果网关之前不在线或者超过了更新间隔，设置为在线
                setOnline(ctx);
             //   lastOnlineTimestamp = currentTime;
            }
            // 确保调用此方法以重置空闲状态监测
          //  ReferenceCountUtil.release(msg); // 如果msg是ByteBuf类型，释放它
           // super.channelRead(ctx, msg);
           // ctx.fireChannelRead(msg);
            // 忽略心跳消息，不传递给下一个handler
        }
        //else {
            // 不是心跳消息，传递给下一个handler
            super.channelRead(ctx, msg);
       // }
        // 此调用保证IdleStateHandler能够重置其读取计时器
      //  ctx.fireChannelRead(msg);
    }

    private boolean isHeartbeatMessage(Object msg) {
        // 判断消息是否为心跳消息
        if (msg instanceof ByteBuf) {
            ByteBuf data = (ByteBuf) msg;
            // 检查数据包是否足够长
            if (data.readableBytes() >= 1) {
                // 标记readerIndex的当前位置，之后可以重置
                data.markReaderIndex();
                // 读取第一个字节
                byte firstByte = data.readByte();
                // 重置readerIndex到标记的位置，这样不会影响后续的读取操作
                data.resetReaderIndex();
                // 检查第一个字节是否为心跳包的标志
                return (firstByte == (byte) 0xFE || firstByte == (byte) 0xfe);
            }
        }
        return false;
    }

    private void setOnline(ChannelHandlerContext ctx) {
        // 更新网关状态为在线
        System.out.println("设备上线");
        isOnline = true;
        // 实现更新状态的逻辑，比如更新数据库记录
    }

    private void setOffline(ChannelHandlerContext ctx) {
        // 更新网关状态为离线
        System.out.println("设备离线");
        isOnline = false;
        // 实现更新状态的逻辑，比如更新数据库记录
        ctx.close(); // 关闭连接
    }
}
