package com.hive.util;

import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;

public class DateConvetUtil {

    public long dateStrToSecond(String dateTimeStr) {
        // 定义日期时间格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 解析日期时间字符串为 LocalDateTime
        LocalDateTime localDateTime = LocalDateTime.parse(dateTimeStr, formatter);

        // 转换为时间戳（秒数）
        long timestamp = localDateTime
                .atZone(ZoneId.systemDefault()) // 获取默认时区
                .toInstant()                   // 转换为 Instant
                .getEpochSecond();               // 转换为秒数
        //.toEpochMilli() 毫秒
        return timestamp;
    }

    public String secondToDateStr(long timestampInSeconds) {
        // 使用 Instant 解析秒级时间戳
        Instant instant = Instant.ofEpochSecond(timestampInSeconds);
        // 转换为 LocalDateTime
        LocalDateTime dateTime = LocalDateTime.ofInstant(instant, ZoneId.systemDefault());
        // 定义输出格式
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 格式化为字符串
        String formattedDateTime = dateTime.format(formatter);
        return formattedDateTime;
    }

    /**
     * 判断两个秒数之间的差异是否大于10分钟
     *
     * @param seconds1 第一个时间的秒数
     * @param seconds2 第二个时间的秒数
     * @return 如果差异大于10分钟返回true，否则返回false
     */
    public static boolean isDifferenceMoreThan10Minutes(long seconds1, long seconds2) {
        long difference = Math.abs(seconds1 - seconds2); // 计算绝对差异
        return difference >= 600; // 10分钟等于600秒
    }

    public static long getCurrentSeconds() {
        // 获取当前时间的秒数
        long currentSeconds = Instant.now().getEpochSecond();
        // 加10分钟 (10分钟 = 600秒)
        long newSeconds = currentSeconds + Duration.ofMinutes(10).getSeconds();
        return newSeconds;
    }

}
